/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * 
* PINEWOOD DERBY ELAPSED TIME TIMER DISPLAY                      PWD_ETDisplay06  Version 2b  02/26/2019 *
* Created by: W. van Leeuwen (Billv923@outlook.com)                                                      *
*                                                                                                        *
* This sketch provides the display interface for the 6-Lane Pinewood Derby Elapsed Time (ET) Timer. It   *
* works in conjunction with the Pinewood Derby timer software resident on the Arduino UNO(tm) micro      *
* controller board. The Arduino UNO provides the hardware interface for the lane finish line sensors and *
* gate (start) switch, performs track management functions and makes the lane time calculations that are *
* sent via the serial (USB) interface to the connected PC running this sketch.                           *
*                                                                                                        *
* NOTE - This version operates in conjunction with Arduino software PWD_ETTimer06-1b                     *
*                                                                                                        *
* This sketch performs the following: (1) receive, process and display track status; (2) receive,        *
* process and display the lane finish times and lane finish sequence; (3) perform race management and    *
* control functions including ability to view prior ran race results and ability to rerun heats,         *
* (4) automatically save all race results to a comma-delimited (.csv) file.                              *
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
   
  import processing.serial.*;               //Use the serial library
  import static javax.swing.JOptionPane.*;  //Use the option pane dialog box library

// Initialization
  int i, j, k, x, y;                        //working integer variables
  int Yr, Mo, Day, Hr, Min, Sec;            //Integer variables for creating date-time based filename
  int Num_Lanes = 6;                        //Set the number of lanes
  int Race_State = 0;                       //0=Not Ready, 1=Ready, 2=Racing, 3=Finished
  int Heat_Number = 0;                      //Heat number selected/displayed
  int Heats_Ran = 0;                        //Number of heats ran
  int Max_Heats = 99;                       //Maximum number of heats allowed
  int Tablerow = -1;                        //Results table row index pointer
  int lf = 10;                              //ASCII Linefeed
  int Index1;                               //Table index pointer
  int [] Place = new int[6];                //Array for storing lane finish order
  int Rx, Gx, Bx;                           //Background color variables
  boolean Rerunflag = false;                //Used to indicate when Rerun Button is to be active
  boolean NextHeatflag = true;              //Used to indicate when Next Heat button is to be active
  float [] Time = new float[6];             //Array for storing lane time values
  String Str1 = "";                         //Working string variable
  String COMx, COMlist = "";                //COM Port check/select routine variables
  String Portname = "";                     //COM Port name variable
  String Filename = "";                     //String variable for the save data file name
  String Time_String = "";                  //String variable for temp storage of time values
  String Msg1 = "";                         //String variable to hold lane sensor error message
  String Msg1a = "Lane sensor(s)";
  String Msg1b = " not ready. Fix problem & click Timer Reset";
  String Msg2 = "Gate (Start) switch not ready, Fix problem & click Timer Reset";
  String Msg3 = "Click Next Heat button to prep for next race";
  String Msg4 = "Click Timer Reset to run a practice heat or Next Heat button to prep for first official heat";
  String Msg5 = "Remote RESET detected";
  String prompt = "";                       //Variable used for storing popup message text
  String inString = "";                     //Input string from serial port
  PFont font;
  Table Resultstable;                       //Define table named 'Resultstable'
  PImage logo;                              //Define an image variable as 'logo'
  
  Serial myPort;                            //Set the Serial port
  
void setup() {
  size(1200, 670);                          //Set display area in pixels
  Rx = 0;                                   //Set background color red hue
  Gx = 240;                                 //Set background color green hue
  Bx = 255;                                 //Set background color blue hue
  background(Rx, Gx, Bx);                   //Set display area background color
  logo = loadImage("BillV_Logo.png");       //Load "Bill V's Derby Timers" Logo
  image(logo, 1080, 610);                   //And display it in lower right corner
  
  //Create unique date-time based file name so that a new file is created each time the program is run.
  //This will prevent previous files from being accidentally overwritten. Note that by default the file
  //is saved to the same folder/directory that contains this sketch.
  Yr = year();                              //Get year value from PC clock
  Mo = month();                             //Get month value from PC clock
  Day = day();                              //Get day value from PC clock
  Hr = hour();                              //Get hour value from PC clock
  Min = minute();                           //Get minute value from PC clock
  Sec = second();                           //Get second value from PC clock
  Filename = "Race_Results_"+nf(Yr,4)+nf(Mo,2)+nf(Day,2)+nf(Hr,2)+nf(Min,2)+nf(Sec,2)+".csv";
  
  //Initialize the 'Resultstable'. The table is used to save data from each heat to a data file
  Resultstable = new Table();               //Name the new table
  Resultstable.addColumn("Heat");           //Add a column named Heat
  Resultstable.addColumn("Lane1");          //Add a column named Lane1
  Resultstable.addColumn("Lane2");          //Add a column named Lane2
  Resultstable.addColumn("Lane3");          //Add a column named Lane3
  Resultstable.addColumn("Lane4");          //Add a column named Lane4
  Resultstable.addColumn("Lane5");          //Add a column named Lane5
  Resultstable.addColumn("Lane6");          //Add a column named Lane6
  
  //Set the font
  // The font must be located in the sketch's "data" directory to load successfully
  font = loadFont("ArialNarrow-Bold-48.vlw");
  textFont(font, 60);
  
  // Draw display header and legend info
  textAlign(CENTER);                         //Set text alignment to center
  fill(255, 0, 170);                         //Set Title color
  text("Organization Name Goes Here",600,50);      //Title - To be updated by end user
  textSize(40);                              //Set text size to 40 pixels
  fill(0);                                   //Set color to black
  text("Lane 1",100,90);                     //Lane 1 legend
  text("Lane 2",300,90);                     //Lane 2 legend
  text("Lane 3",500,90);                     //Lane 3 legend
  text("Lane 4",700,90);                     //Lane 4 legend
  text("Lane 5",900,90);                     //Lane 5 legend
  text("Lane 6",1100,90);                    //Lane 6 legend
  textSize(30);                              //Set text size to 30 pixels
  textAlign(LEFT);                           //Set text alignment to left
  fill(Rx, Gx, Bx);                          //Set color same as background
  rect(40,390,260,190,10);                   //Draw rectangle for Race Status box
  rect(320,390,840,190,10);                  //Draw rectangle for Race Control box
  rect(320,390,100,190,10);                  //Draw rectangle for Reset Timer box
  fill(0);                                   //Set color back to black
  text("Race Status",100,420);               //Draw Race Status box legend
  text("Race Control",700,420);              //Draw Race Control box legend
  text("Messages:",40,610);                  //Draw message box legend
  fill(255, 0, 0);                           //Set color to Red
  ellipse(370,500,50,50);                    //Draw Reset Button
  textAlign(CENTER);                         //Set text alignment to center
  textSize(25);                              //Set text size to 25 pixels
  text("Timer",370,470);                     //Draw Reset Button legend
  text("Reset",370,550);                     //Draw Reset Button legend
  fill(0,0,255);                             //Set color to Blue
  ellipse(750,500,50,50);                    //Draw Terminate Race button
  text("Terminate",750,470);                 //Draw Terminate Race button legend
  text("Race",750,550);                      //Draw Terminate Race button legend
  fill(0);                                   //Set color back to black
  text("Heat #",1060,455);                   //Draw Heat Box text header
  fill(100);                                 //Set rectangle color
  rect(1110,460,30,80);                      //and draw Up/Down increment box
  fill(0,0,255);                             //Set color to Blue
  triangle(1110, 495, 1140, 495, 1125, 465); //Draw Up Arrow
  triangle(1110, 505, 1140, 505, 1125, 535); //Draw Down Arrow
  
  //Initialize text & message boxes by calling their respective subroutines
  Clear_Data();                              //Subroutine to initialize lane time text boxes
  Clear_Msgbox();                            //Subroutine to initialize Message box
  Draw_Heatbox();                            //Subroutine to initialize Heat Count box
  Set_Race_State();                          //Subroutine to initialize Race Status box
  Draw_NextHeat_Button();                    //Call subroutine to draw the Next Heat button
  Draw_RerunHeat_Button();                   //Call subroutine to draw the Rerun Heat button
  
  //Check & Open the serial port at 9600 baud and buffer data till linefeed received
  try {                                      //Use the 'try' & 'catch' exception handling process
    i = Serial.list().length;                //Get the number of open ports
    if (i != 0) {                            //Do if at least one port is open
      if (i >= 2) {                          //Do if more than one open port
        //Create a list of found COM ports from which the user can pick
        for (j = 0; j < i; j++) {
          COMlist += char(j+'a') + " = " + Serial.list()[j];
          if (j+1 < i) COMlist += ",  ";
        }
        COMx = showInputDialog("Which COM port is correct? (a,b,..):\n"+COMlist);
        if (COMx == null) exit();
        if (COMx.isEmpty()) exit();
        i = int(COMx.toLowerCase().charAt(0) - 'a') + 1;
      }
      Portname = Serial.list()[i-1];               //Get the selected port name
      myPort = new Serial(this, Portname, 9600);   //Activate COM port at 9600 baud
      myPort.bufferUntil(lf);                      //Buffer until LF appears
      prompt = "COM Port Selected: " + Portname;   //Construct COM message & COM Port
      showMessageDialog(frame,prompt);             //and display COM Port message
    }
    else {
      showMessageDialog(frame,"No open/connected COM ports detected.\nFix problem & restart program");
      exit();
    }
  }
  catch (Exception e) { //Error catch routine. Print the type of error
    showMessageDialog(frame,"COM port is not available (may\nbe in use by another program)");
    println("Error:", e);
    exit();
  }
  Disp_Message(4);                            //Display initial race start message
} //End of "Setup" routine

//********************************************************************************************************
void draw() {  //Note: Things aren't displayed on the screen until the draw loop completes.
  //Get & process serial data from the Arduino
  if (inString.indexOf("NRD") >= 0) {        //Check for receipt of 'Not Ready' state message
    Race_State = 0;                          //Set the race state to 0 (0=not ready)
    Set_Race_State();                        //and call subroutine to display the race state
    if (Heat_Number > 0) {                   //Don't allow if no heats have been ran yet
      NextHeatflag = false;                  //Disable Next Heat button
      Draw_NextHeat_Button();                //Call subroutine to update Next Heat button display
    }
    Rerunflag = false;                       //Set Rerunflag to disable Rerun Heat button
    Draw_RerunHeat_Button();                 //Call subroutine to update Rerun Heat button display
    inString = "";                           //Clear 'Not Ready' message once processed
  }
  if (inString.indexOf("RDY") >= 0) {        //Check for receipt of 'Ready' state
    Race_State = 1;                          //Set the race state to 1 (1=ready)
    Clear_Data();                            //Call subroutine to clear data from text boxes
    Clear_Msgbox();                          //Call subroutine to initialize/clear Message box
    Set_Race_State();                        //Call subroutine to display the race state
    if (Heat_Number > 0) {                   //Don't allow if no heats have been ran yet
      NextHeatflag = false;                  //Disable Next Heat button
      Draw_NextHeat_Button();                //Call subroutine to update Next Heat button display
    }
    Rerunflag = false;                       //Set Rerunflag to disable Rerun Heat button
    Draw_RerunHeat_Button();                 //Call subroutine to update Rerun Heat button display
    inString = "";                           //Clear 'Ready' message once processed
  }
  if (inString.indexOf("RAC") >= 0) {        //Check for receipt of 'Racing' state
    Race_State = 2;                          //Set the race state to 2 (2=racing)
    Set_Race_State();                        //and call subroutine to display the race state
    inString = "";                           //Clear 'Racing' message once processed
  }
  if (inString.indexOf("FIN") >= 0) {        //Check for receipt of 'Finished' state
    Race_State = 3;                          //Set the race state to 3 (3=finished)
    Set_Race_State();                        //and call subroutine to display the race state
    if (Heat_Number == 0) Disp_Message(4);   //Set pointer to alternate message if no heats run
    else Disp_Message(3);                    //Display message
    Rerunflag = true;                        //Set Rerunflag to enable Rerun Heat button
    Draw_RerunHeat_Button();                 //Call subroutine to update Rerun Heat button display
    NextHeatflag = true;                     //Enable Next Heat button
    Draw_NextHeat_Button();                  //Call subroutine to update Next Heat button display
    inString = "";                           //Clear 'Finished' message once processed
  }
  if (inString.indexOf("GSW") >= 0) {        //Check for receipt of Gate Switch error code
    Disp_Message(2);                         //Display Gate Switch error message
    inString = "";                           //Clear error message once processed
  }
  if (inString.indexOf("TRK") >= 0) {        //Check for receipt of Track Sensor error code
    i = inString.length() - 2;               //Get length of inString minus cr/lf characters
    Str1 = inString.substring(4, i);         //Extract lane sensor numbers from inString
    Msg1 = Msg1a + Str1 + Msg1b;             //Construct the error message to display
    Disp_Message(1);                         //and display track sensor error message
    inString = "";                           //Clear error message once processed
  }
  if (inString.indexOf("RST") >= 0) {        //Check for receipt of Remote RESET message
    Disp_Message(5);                         //Display Remote Reset message
    inString = "";                           //Clear message once processed
  }
  if (inString.indexOf("Time") >= 0) {       //Check for receipt of Race times string
    Time_String = inString;                  //and save it to a temp holding variable
    //Note: This string has the following format: "Time: a.aaa b.bbb c.ccc d.ddd e.eee f.fff"
    //      where a.aaa through f.fff are the lane times for lanes 1 - 6 respectively
    Str1 = Time_String.substring(7, 13);     //Extract race time string for lane 1
    Time[0] = float (Str1);                  //and convert it to a floating point number
    Str1 = Time_String.substring(14, 20);    //Extract race time string for lane 2
    Time[1] = float (Str1);                  //and convert it to a floating point number
    Str1 = Time_String.substring(21, 27);    //Extract race time string for lane 3
    Time[2] = float (Str1);                  //and convert it to a floating point number
    Str1 = Time_String.substring(28, 34);    //Extract race time string for lane 4
    Time[3] = float (Str1);                  //and convert it to a floating point number
    Str1 = Time_String.substring(35, 41);    //Extract race time string for lane 5
    Time[4] = float (Str1);                  //and convert it to a floating point number
    Str1 = Time_String.substring(42, 48);    //Extract race time string for lane 6
    Time[5] = float (Str1);                  //and convert it to a floating point number
    inString = "";                           //Clear Race times string once processed
    
    //Determine the lane finish order (place) based on finish times
    for (i=0; i<Num_Lanes; i++) {            //First set all Place holders to last (4'th) place
      Place[i] = Num_Lanes;
    }
    for (i=0; i<Num_Lanes; i++) {            //Now compare each time to the other times and
      for (j=0; j<Num_Lanes; j++) {          //decrement its place value each time it is less
        if (Time[i] < Time[j]) {             //A value of 9.9999, if present, gets set to last place thus
          Place[i]--;                        //maintaining correct finish order for the valid lanes
        }
      }
    }
    Display_Results();                       //Call subroutine to display the results
  
    //Write the time values to the Resultstable
    if (Heat_Number > Heats_Ran) {           //Do only if a new heat
      Heats_Ran = Heat_Number;               //Bump the Heats Ran count
      Resultstable.addRow();                 //Add new row to the table
    }
    if (Heat_Number > 0) {                   //Don't save if practice run (i.e. Heat Number = #)
      Tablerow = Heat_Number - 1;            //Set the table row index
      Resultstable.setInt(Tablerow, "Heat", Heat_Number);   //Save Heat Number
      Resultstable.setFloat(Tablerow, "Lane1", Time[0]);    //Save Lane1 time
      Resultstable.setFloat(Tablerow, "Lane2", Time[1]);    //Save Lane2 time
      Resultstable.setFloat(Tablerow, "Lane3", Time[2]);    //Save Lane3 time
      Resultstable.setFloat(Tablerow, "Lane4", Time[3]);    //Save Lane4 time
      Resultstable.setFloat(Tablerow, "Lane5", Time[4]);    //Save Lane5 time
      Resultstable.setFloat(Tablerow, "Lane6", Time[5]);    //Save Lane6 time
      //Save (re-save) all results to the datafile each time new results are available. Note that by default
      //the file is saved to the same folder/directory that contains this sketch.
      saveTable(Resultstable, Filename);
    }
    delay(100);                                      //Wait awhile before we continue
  }
} //End of "draw" loop

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
*                                            SUBROUTINES                                               *
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void serialEvent(Serial p) {                  //Serial data handling routine
  inString = p.readString();
}

void mouseClicked() {                        //Mouse Click handling routine
  //This routine is called when the mouse button is clicked. Mouse X & Y coordinates are tested to see
  //if the mouse pointer is over the RESET button, SAVE RESULTS button, HEAT Increment/Decrement buttons,
  //or one of the lane mask select check boxes. Appropriate action is then taken.
  x = mouseX;                                //Get mouse X-coordinate
  y = mouseY;                                //Get mouse Y-coordinate
  //Test for RESET button and send the reset command to the Arduino if so.
  if ((x>345) && (x<395) && (y>475) && (y<525)) {
    myPort.write("R");                       //Send the letter R
    myPort.write(lf);                        //followed by a line-feed
    NextHeatflag = true;                     //Enable Next heat button (in case it was disabled)
    Draw_NextHeat_Button();                  //And update display
  }
  //Test for Next Heat select button and perform necessary actions
  if ((x>460) && (x<540) && (y>470) && (y<530)) {
    if (NextHeatflag && (Heats_Ran<Max_Heats)) {  //Only allow if next heat flag is TRUE & heats ran < 99
      Heat_Number = Heats_Ran + 1;           //And set the heat number
      Draw_Heatbox();                        //Redisplay new heat value
      myPort.write("R");                     //Send the Reset command to Arduino to ready the timer
      myPort.write(lf);                      //followed by a line-fee
    }
  }
  //Test for Rerun Heat select button and toggle Rerunflag
  if ((x>610) && (x<660) && (y>470) && (y<530)) {
    if (Rerunflag) {                                 //Only allow if Rerun flag is TRUE
      Str1 = "RERUN SELECT";                         //Rerun heat select dialog header
      prompt = "RERUN CURRENTLY SELECTED HEAT\nAre you sure?";   //Build the prompt string
      int input = showConfirmDialog(null, prompt, Str1, YES_NO_OPTION); //Prompt user for heat rerun: 0=yes, 1=no
      if (input == 0) {                              //Check if YES is selected and do following
        //Heat_Number = Heats_Ran;                   //Set the heat number to last heat ran
        //Draw_Heatbox();                            //Redisplay new heat value
        myPort.write("R");                           //Send the Reset command to Arduino to ready the timer
        myPort.write(lf);                            //followed by a line-feed
      }
    }
  }
  //Test for Terminate Race button and end program if so.
  if ((x>725) && (x<775) && (y>475) && (y<525)) {
    Str1 = "TERMINATE RACE SELECT";                  //Terminate program select dialog header
    prompt = "TERMINATE RACE\nAre you sure?";        //Build the prompt string
    int input = showConfirmDialog(null, prompt, Str1, YES_NO_OPTION); //Prompt for race termin
    if (input == 0) {                                //Check if YES is selected and do following
      delay(100);
      prompt = "RACE TERMINATED\nResults have been saved\nRestart program to run a new race";
      showMessageDialog(frame,prompt);               //Display Dialog
      exit();
    }
  }
  //Test for Up Arrow button and increment Heat Number value by one if so.
  if ((x>1110) && (x<1140) && (y>465) && (y<495)) {
    if (Heat_Number==Heats_Ran && Race_State==1) {
      Heat_Number++;                         //Increment Heat number if ready for next race
      Draw_Heatbox();                        //and redisplay new value
      Clear_Data();                          //Clear data fields to ready for next race
    }
    if (Heat_Number < Heats_Ran) {
      Heat_Number++;                         //Increment Heat if < number of heats ran
      Draw_Heatbox();                        //and redisplay new value
      Index1 = Heat_Number - 1;              //Set table index pointer
      Redisplay_RaceResults();               //Call subroutine to redisplay results
    }
  }
  //Test for Down Arrow button and decrement Heat Number value by one if so.
  if ((x>1110) && (x<1140) && (y>505) && (y<535)) {
    if (Heat_Number > 1) {
      Heat_Number--;                         //Decrement Heat if < 1
      Draw_Heatbox();                        //and redisplay new value
      Index1 = Heat_Number - 1;              //Set table index pointer
      Redisplay_RaceResults();               //Call subroutine to redisplay results
    }
  }
}

void Clear_Data() {
  //Draw text boxes with curved corners in which the finish order and lane times will be displayed.
  //Note: This will also overwrite (clear) any previous data drawn inside
  fill(100);                                 //Set rectangle color
  for (i=0; i<Num_Lanes; i++) {              //Loop through all 6 lanes
    rect(65+i*200,100,70,120,10);            //Draw rectangles for lane finish order
    rect(10+i*200,240,180,60,10);            //Draw rectangles for lane time
  }
}

void Clear_Msgbox() {
  //Draw rectangle message box in which the error messages will be displayed
  //Note: This will also overwrite (clear) any previous data drawn inside
  fill(100);                                 //Set rectangle color
  rect(40,620,1020,40,10);                   //Draw it
}

void Draw_Heatbox() {
  //Draw the Heat Count box and current Heat number
  //Note: This will also overwrite (clear) any previous data drawn inside the box
  fill(100);                                 //Set rectangle color
  rect(980,460,130,80);                      //and draw it
  textSize(65);                              //Set text size to 65 pixels
  textAlign(CENTER);                         //Set text alignment to center
  fill(255);                                 //Set text color to white
  if (Heat_Number > 0) text(Heat_Number,1045,525);
    else text("#",1045,525);
}

void Set_Race_State() {
  textAlign(LEFT);                           //Set text alignment to left
  textSize(25);                              //Set text size to 25 pixels
  fill(200);                                 //Set color to dim circle & text
  ellipse(80,455,25,25);                     //Draw dimmed circle
  text("Track Not Ready",110,465);           //Draw dimmed text
  ellipse(80,490,25,25);                     //Draw dimmed circle
  text("Ready",110,500);                     //Draw dimmed text
  ellipse(80,525,25,25);                     //Draw dimmed circle
  text("Racing",110,535);                    //Draw dimmed text
  ellipse(80,560,25,25);                     //Draw dimmed circle
  text("Finished",110,570);                  //Draw dimmed text
  switch (Race_State) {                      //Test the race state
    case 0:                                  //Do the following if it is 0
      fill(255,0,0);                         //Set color to red
      ellipse(80,455,25,25);                 //Draw red circle
      text("Track Not Ready",110,465);       //Draw red text
      break;
    case 1:                                  //Do the following if it is 1
      fill(245, 255, 0);                     //Set color yellow
      ellipse(80,490,25,25);                 //Draw yellow circle
      text("Ready",110,500);                 //Draw yellow text
      break;
    case 2:                                  //Do the following if it is 2
      fill(0, 180, 0);                       //Set color to dark green
      ellipse(80,525,25,25);                 //Draw green circle
      text("Racing",110,535);                //Draw green text
      break;
    case 3:                                  //Do the following if it is 3
      fill(0, 0, 200);                       //Set color to blue
      ellipse(80,560,25,25);                 //Draw blue circle
      text("Finished",110,570);              //Draw blue text
      break;
    default:                                 //Do nothing if it is something else
      //do nothing
      break;
  }
}

void Display_Results() {
  Clear_Data();                                   //Call subroutine to clear time & finish order
  //Write the time values into their respective text boxes on the screen
  textSize(65);                                   //Set text size to 65 pixels
  textAlign(CENTER);                              //Set text alignment to center
  fill(255);                                      //Set text color to white
  Str1=nf(Time[0],1,4);                           //Convert so it won't round to 3 decimal places
  if (Time[0] != 9.9999) text(Str1,105,295);      //Check for a valid time value and write if so
    else text("- - - - -",100,290);               //Or write dashes into the lane 1 text box
  Str1=nf(Time[1],1,4);                           //Convert so it won't round to 3 decimal places
  if (Time[1] != 9.9999) text(Str1,305,295);      //Check for a valid time value and write if so
    else text("- - - - -",300,290);               //Or write dashes into the lane 2 text box
  Str1=nf(Time[2],1,4);                           //Convert so it won't round to 3 decimal places
  if (Time[2] != 9.9999) text(Str1,505,295);      //Check for a valid time value and write if so
    else text("- - - - -",500,290);               //Or write dashes into the lane 3 text box
  Str1=nf(Time[3],1,4);                           //Convert so it won't round to 3 decimal places
  if (Time[3] != 9.9999) text(Str1,705,295);      //Check for a valid time value and write if so
    else text("- - - - -",700,290);               //Or write dashes into the lane 4 text box
  Str1=nf(Time[4],1,4);                           //Convert so it won't round to 3 decimal places
  if (Time[4] != 9.9999) text(Str1,905,295);      //Check for a valid time value and write if so
    else text("- - - - -",900,290);               //Or write dashes into the lane 5 text box
  Str1=nf(Time[5],1,4);                           //Convert so it won't round to 3 decimal places
  if (Time[5] != 9.9999) text(Str1,1105,295);     //Check for a valid time value and write if so
    else text("- - - - -",1100,290);              //Or write dashes into the lane 6 text box
    
  //Write the finish order values into their respective text boxes on the screen if valid
  textSize(135);                                  //Set text size to 135 pixels
  if (Time[0] != 9.9999) text(Place[0],100,210);  //Write Place value for Lane 1 if it is valid
  if (Time[1] != 9.9999) text(Place[1],300,210);  //Write Place value for Lane 2 if it is valid
  if (Time[2] != 9.9999) text(Place[2],500,210);  //Write Place value for Lane 3 if it is valid
  if (Time[3] != 9.9999) text(Place[3],700,210);  //Write Place value for Lane 4 if it is valid
  if (Time[4] != 9.9999) text(Place[4],900,210);  //Write Place value for Lane 5 if it is valid
  if (Time[5] != 9.9999) text(Place[5],1100,210); //Write Place value for Lane 6 if it is valid
}

void Redisplay_RaceResults() {     //Get & Redisplay time & finish order for the current selected heat
  //Read times from results table
  Time[0] = Resultstable.getFloat(Index1, "Lane1"); //Get recorded car time for Lane 1
  Time[1] = Resultstable.getFloat(Index1, "Lane2"); //Get recorded car time for Lane 2
  Time[2] = Resultstable.getFloat(Index1, "Lane3"); //Get recorded car time for Lane 3
  Time[3] = Resultstable.getFloat(Index1, "Lane4"); //Get recorded car time for Lane 4
  Time[4] = Resultstable.getFloat(Index1, "Lane5"); //Get recorded car time for Lane 5
  Time[5] = Resultstable.getFloat(Index1, "Lane6"); //Get recorded car time for Lane 6
  //Determine the lane finish order (place) based on finish times
  for (i=0; i<Num_Lanes; i++) {                 //First set all Place holders to last (6'th) place
    Place[i] = Num_Lanes;
  }
  for (i=0; i<Num_Lanes; i++) {                 //Now compare each time to the other times and
    for (j=0; j<Num_Lanes; j++) {               //decrement its place value each time it is less
      if (Time[i] < Time[j]) {                  //A value of 9.999, if present, gets set to last place thus
        Place[i]--;                             //maintaining correct finish order for the valid lanes
      }
    }
  }
  Display_Results();                            //Call subroutine to display the results
}

void Draw_NextHeat_Button() {
  fill(200);                                 //Set rectangle color
  rect(460,470,80,60,10);                    //Draw rectangle
  textSize(25);                              //Set text size to 25 pixels
  textAlign(CENTER);                         //Set text alignment to center
  fill(0);                                   //Set text color to black
  if (!NextHeatflag) fill(150);              //Set text color to grey (Disabled) if NextHeatflag = FALSE
  text("Next",500,495);                      //Display "Next" inside text box top
  text("Heat",500,525);                      //Display "Heat" inside text box bottom
}

void Draw_RerunHeat_Button() {               //Subroutine to display the Rerun Heat select button
  fill(200);                                 //Set rectangle color
  rect(580,470,80,60,10);                    //Draw rectangle
  textSize(25);                              //Set text size to 25 pixels
  textAlign(CENTER);                         //Set text alignment to center
  fill(0);                                   //Set text color to black
  if (!Rerunflag) fill(150);                 //Set text color to grey (Disabled) if Rerunflag = FALSE
  text("Rerun",620,495);                     //Display "Next" inside text box top
  text("Heat",620,525);                      //Display "Heat" inside text box bottom
}

void Disp_Message(int Msg_Number) {
  Clear_Msgbox();                            //Call Clear Message Box subroutine
  textAlign(LEFT);                           //Set text alignment to left
  textSize(23);                              //Set text size to 23 pixels
  fill(255);                                 //Set color to white
  switch (Msg_Number) {                       //Test the message state
    case 0:                                  //Do nothing if it is 0
      //do nothing
      break;
    case 1:                                  //Do the following if it is 1
      text(Msg1,50,647);                     //Display message
      break;
    case 2:                                  //Do the following if it is 2
      text(Msg2,50,647);                     //Display message
      break;
    case 3:                                  //Do the following if it is 3
      text(Msg3,50,647);                     //Display message
      break;
    case 4:                                  //Do the following if it is 4
      text(Msg4,50,647);                     //Display message
      break;
    case 5:                                  //Do the following if it is 5
      text(Msg5,50,647);                     //Display message
      break;
    default:                                 //Do nothing if it is something else
      //do nothing
      break;
  }
}
